# This file is part of Matt.

# Copyright © 2020 Noisytoot
# MATT - MATT Arithmetic Training Test: Another maths test, this time in Python!

# Matt is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Matt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this Matt.  If not, see <https://www.gnu.org/licenses/>

import argparse, secrets, platform, sys, signal, os
# Import readline except on windows
if platform.system() != 'Windows':
    import readline
from matt.termcolor_wrapper import colored, cprint
from xdg.BaseDirectory import xdg_config_home
from pathlib import Path
config_file = os.path.join(xdg_config_home, "matt", "config.py")
config = {}
try:
    exec(Path(config_file).read_text(), config)
except FileNotFoundError:
    pass

# Initiali(s|z)e colorama (ANSI escape emulation on windows) (windows only)
if platform.system() == "Windows":
    import colorama
    colorama.init()

# Parse command line arguments
parser = argparse.ArgumentParser(prog="mat", description="A maths test program")
parser.add_argument("-m", "--minimum", help="The minimum number", type=str)
parser.add_argument("-M", "--maximum", help="The maximum number", type=str)
parser.add_argument("-o", "--operations", help='Allowed operations ("+", "-", "*", "/"), seperated by commas', type=str)
parser.add_argument("-d", "--difficulty", help="Difficulty profile (preset of difficulty related settings)", type=str)
parser.add_argument("-q", "--question-amount", help="Amount of questions", type=int)
args = parser.parse_args()

def sigint_exit(_signal, _frame):
    print("\nSIGINT caught, exiting…")
    sys.exit(0)

def sigterm_exit(_signal, _frame):
    print("\nSIGTERM caught, exiting…")
    sys.exit(0)

signal.signal(signal.SIGINT, sigint_exit)
signal.signal(signal.SIGTERM, sigterm_exit)


def isfloat(number) -> bool:
    try:
        float(number)
        return True
    except ValueError:
        return False

# Default settings for difficulties
def default_difficulty(namespace, number):
    if namespace == "default":
        if number == 1:
            return {
                "operations": ["+", "-"],
                "maximum": 10,
                "minimum": 0
            }
        elif number == 2:
            return {
                "operations": ["+", "-", "*", "/"],
                "maximum": 20,
                "minimum": 0
            }

def do_level(namespace: str, number: int, question_amount: int):
    if "difficulty" in config and config["difficulty"](namespace, number) != None:
        difficulty = config["difficulty"](namespace, number)
    else:
        difficulty = default_difficulty(namespace, number)
    
    # Set value for minimum and maximum from arguments and default
    try:
        difficulty["minimum"] = int(args.minimum)
    except TypeError:
        if "minimum" not in difficulty:
            difficulty["minimum"] = 0
    try:
        difficulty["maximum"] = int(args.maximum)
    except TypeError:
        if "maximum" not in difficulty:
            difficulty["maximum"] = difficulty["minimum"] + 10
    try:
        difficulty["operations"] = args.operations.split(",")
    except AttributeError:
        if "operations" not in difficulty:
            difficulty["operations"] = ["+", "-", "*", "/"]
    
    score: float = 0
    question: int = 0
    while question < question_amount:
        # Generate random operation
        operation: str = secrets.choice(difficulty["operations"])
        
        # Generate 2 random numbers
        try:
            n1 = secrets.choice(range(difficulty["minimum"], difficulty["maximum"]))
            n2 = secrets.choice(range(difficulty["minimum"], difficulty["maximum"]))
        except IndexError:
            message = f"""\
Fatal Error: minimum cannot be equivalent to maximum
minimum = {difficulty["minimum"]}
maximum = {difficulty["maximum"]}\
"""
            sys.exit(colored(message, "red", attrs=["bold"]))
        if operation == "/":
            divanswer = n1
            n1 = divanswer * n2
        
        # Set correct answer
        if operation == "+":
            correct_answer = n1 + n2
        elif operation == "-":
            correct_answer = n1 - n2
        elif operation == "*":
            correct_answer = n1 * n2
        elif operation == "/":
            correct_answer = divanswer
        
        # Get the user's answer
        answer = ""
        while not isfloat(answer):
            answer = input(colored(f"What is {n1} {operation} {n2}? ", "cyan", attrs=["bold"]))
        answer = float(answer)
        
        # Check the user's answer
        if correct_answer == answer:
            cprint("Correct!", "green", attrs=["bold"])
            score += 1
        elif abs(correct_answer - answer) <= 2:
            points = 0.5 - 4 / difficulty["maximum"]
            cprint(f"Not quite right, the correct answer was {correct_answer}, you get {points} points.", "yellow", attrs=["bold"])
            score += points
        else:
            cprint(f"Wrong, the correct answer was {correct_answer}.", "red", attrs=["bold"])
            question_amount += 1
        
        # Increment the question
        question += 1
    cprint(f"Your score was {score}/{question_amount}.", "yellow", attrs=["bold"])

try:
    difficulty: list = args.difficulty.split(":")
except AttributeError:
    cprint("Warning: difficulty not set, defaulting to default:1", "red", file=sys.stderr)
    difficulty: list = ["default", 1]

try:
    question_amount: int = int(args.question_amount)
except TypeError:
    cprint("Warning: amount of questions not set, defaulting to 10", "red", file=sys.stderr)
    question_amount: int = 10

do_level(difficulty[0], int(difficulty[1]), question_amount)

# Deinitiali(s|z)e colorama (windows only)
if platform.system() == "Windows":
    colorama.deinit()



# This file is part of Matt

# Copyright © 2020 Noisytoot
# MATT - MATT Arithmetic Training Test: Another maths test, this time in Python!

# Matt is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Matt is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Matt.  If not, see <https://www.gnu.org/licenses/>

import os, termcolor
__ALL__ = ["colored", "cprint"]
if "NO_COLOR" in os.environ:
    os.environ["ANSI_COLORS_DISABLED"] = "1"
colored = termcolor.colored
cprint = termcolor.cprint



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['matt']

package_data = \
{'': ['*']}

install_requires = \
['pyxdg>=0.26,<0.27', 'termcolor>=1.1.0,<2.0.0']

extras_require = \
{':platform_system != "Windows"': ['readline>=6.2.4,<7.0.0'],
 ':sys_platform == "windows"': ['colorama>=0.4.3,<0.5.0']}

setup_kwargs = {
    'name': 'matt',
    'version': '0.1.1',
    'description': 'A maths test',
    'long_description': '# Matt\nMatt is a free software (licensed under the [GNU GPL v3 (or later)](https://www.gnu.org/licenses/gpl-3.0.html)) maths test program.\n"Matt" (or "MATT") is a recursive acronym for "MATT Arithmetic Training Test".\n\n## Installation\nMatt depends on:\n * [termcolor](https://pypi.org/project/termcolor/)\n * [pyxdg](https://www.freedesktop.org/wiki/Software/pyxdg/)\n * [colorama](https://pypi.org/project/colorama/) (windows only)\n\nThere are 2 methods of installation: via poetry and via pip:\n\n### Installation via [poetry](https://python-poetry.org/)\nPoetry should handle the installation of dependencies.\nTo install, first clone the git repository, then run:\n```sh\npoetry install\n```\n\n### Installation via pip\nPip should handle the installation of dependencies.\nTo install, run:\n```sh\npython3 -m pip install matt\n```\n\n## Usage\nRun `python3 -m matt -h` for help.\nMatt accepts the following arguments:\n * `--difficulty` or `-d` to set the difficulty (in the format "<namespace>:<number>").\n   The `default` namespace is reserved for the default difficulties.\n   If unspecified the default is `default:1`.\n * `--operations` or `-o` to set the available operations.\n   Operations are seperated by commas, the available operations are:\n   - `+`: Addition\n   - `-`: Subtraction\n   - `*`: Multiplication\n   - `/`: Division\n   Example: `-o +,-` to enable only addition and subtraction.\n * `--minimum` or `-m` to set the minumum, default (if not specified in difficulty): 0.\n * `--maximum` or `-M` to set the maximum, default (if not specified in difficulty): 10.\nNOTE: The maximum must not be equivalent to the minimum.\n\n## Config\nMatt has a configuration file, it is written in Python,\nand located at `$XDG_CONFIG_HOME/matt/config.py`.\nBy default `$XDG_CONFIG_HOME` is set to `~/.config`,\nso if you have not set it then it is probably `~/.config/matt/config.py`.\n\nThe configuration must provide a difficulty function that accepts 2 parameters:\n * `namespace` (str): The namespace of the difficulty.\n * ``number`` (int): The number of the difficulty.\n\nThe ``difficulty`` function must return a dict.\n\nA simple example is:\n```python\ndef difficulty(namespace: str, number: int) -> dict:\n    if namespace == "manual":\n       if number == 1:\n           return {\n               "operations": ["+", "-"],\n               "maximum": 20,\n               "minimum": 10\n           }\n```\n\nDue to the config file being written in Python,\nit is extemely configurable, for example:\n```python\ndef difficulty(namespace: str, number: int) -> dict:\n    if namespace == "automatic":\n        return {\n            "operations": ["+", "-", "*", "/"],\n            "maximum": number * 10\n        }\n```\n',
    'author': 'Noisytoot',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://codeberg.org/noisytoot/matt',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
